@@ -8,7 +8,7 @@
 int main()
 {
   uint64_t count = primesieve_count_primes(0, 1000);
-  printf("Primes below 1000 = %" PRIu64 "\n", count);
+  printf("Primes <= 1000: %" PRIu64 "\n", count);
 
   return 0;
 }
