@@ -1,7 +1,7 @@
 ///
 /// @file  CpuInfo.hpp
 ///
-/// Copyright (C) 2021 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -10,9 +10,10 @@
 #ifndef CPUINFO_HPP
 #define CPUINFO_HPP
 
+#include "pod_vector.hpp"
+
 #include <cstddef>
 #include <string>
-#include <array>
 
 namespace primesieve {
 
@@ -21,6 +22,7 @@ class CpuInfo
 public:
   CpuInfo();
   bool hasCpuName() const;
+  bool hasAVX512() const;
   bool hasLogicalCpuCores() const;
   bool hasL1Cache() const;
   bool hasL2Cache() const;
@@ -41,8 +43,8 @@ public:
 private:
   void init();
   std::size_t logicalCpuCores_;
-  std::array<std::size_t, 4> cacheSizes_;
-  std::array<std::size_t, 4> cacheSharing_;
+  pod_array<std::size_t, 4> cacheSizes_;
+  pod_array<std::size_t, 4> cacheSharing_;
   std::string error_;
 };
 
