@@ -12,14 +12,15 @@
 
 #include "Bucket.hpp"
 #include "macros.hpp"
-#include "MemoryPool.hpp"
+#include "pod_vector.hpp"
 #include "Wheel.hpp"
 
 #include <stdint.h>
-#include <vector>
 
 namespace primesieve {
 
+class MemoryPool;
+
 /// EratBig is an implementation of the segmented sieve of
 /// Eratosthenes optimized for big sieving primes that have
 /// very few multiples per segment.
@@ -28,17 +29,16 @@ class EratBig : public Wheel210_t
 {
 public:
   void init(uint64_t, uint64_t, uint64_t, MemoryPool&);
-  NOINLINE void crossOff(uint8_t*);
-  bool enabled() const { return enabled_; }
+  NOINLINE void crossOff(pod_vector<uint8_t>& sieve);
+  bool hasSievingPrimes() const { return !buckets_.empty(); }
 private:
   uint64_t maxPrime_ = 0;
   uint64_t log2SieveSize_ = 0;
   uint64_t moduloSieveSize_ = 0;
   MemoryPool* memoryPool_ = nullptr;
-  std::vector<SievingPrime*> buckets_;
-  bool enabled_ = false;
+  pod_vector<SievingPrime*> buckets_;
   void storeSievingPrime(uint64_t, uint64_t, uint64_t);
-  NOINLINE void crossOff(uint8_t*, Bucket*);
+  NOINLINE void crossOff(uint8_t* sieve, SievingPrime* prime, SievingPrime* end);
 };
 
 } // namespace
