@@ -1,7 +1,7 @@
 ///
 /// @file  EratSmall.hpp
 ///
-/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -12,10 +12,11 @@
 
 #include "Bucket.hpp"
 #include "macros.hpp"
+#include "pod_vector.hpp"
 #include "Wheel.hpp"
 
+#include <cstddef>
 #include <stdint.h>
-#include <vector>
 
 namespace primesieve {
 
@@ -26,17 +27,15 @@ namespace primesieve {
 class EratSmall : public Wheel30_t
 {
 public:
-  static uint64_t getL1CacheSize(uint64_t);
   void init(uint64_t, uint64_t, uint64_t);
-  void crossOff(uint8_t*, uint64_t);
-  bool enabled() const { return enabled_; }
+  void crossOff(pod_vector<uint8_t>& sieve);
+  bool hasSievingPrimes() const { return !primes_.empty(); }
 private:
   uint64_t maxPrime_ = 0;
-  uint64_t l1CacheSize_ = 0;
-  std::vector<SievingPrime> primes_;
-  bool enabled_ = false;
+  std::size_t l1CacheSize_ = 0;
+  pod_vector<SievingPrime> primes_;
   void storeSievingPrime(uint64_t, uint64_t, uint64_t);
-  NOINLINE void crossOff(uint8_t*, uint8_t*);
+  NOINLINE void crossOff(uint8_t*, std::size_t);
 };
 
 } // namespace
