@@ -25,21 +25,132 @@
 ///
 
 #include <primesieve/CpuInfo.hpp>
+#include <primesieve/macros.hpp>
 
+#include <algorithm>
 #include <stdint.h>
 #include <cstddef>
 #include <exception>
 #include <string>
 #include <vector>
 
-using std::size_t;
+#if defined(__APPLE__) && \
+    __has_include(<sys/sysctl.h>)
+  #define APPLE_SYSCTL
+#endif
 
-#if defined(__APPLE__)
-  #if !defined(__has_include)
-    #define APPLE_SYSCTL
-  #elif __has_include(<sys/sysctl.h>)
-    #define APPLE_SYSCTL
+#if defined(__i386__) || \
+    defined(__x86_64__) || \
+    defined(_M_IX86) || \
+    defined(_M_X64)
+
+#if defined(_MSC_VER)
+  #include <intrin.h>
+  #include <immintrin.h>
+#endif
+
+#define HAS_CPUID
+
+/* %ebx bit flags */
+#define bit_AVX512F (1 << 16)
+
+/* %ecx bit flags */
+#define bit_AVX512VBMI  (1 << 1)
+#define bit_AVX512VBMI2 (1 << 6)
+#define bit_POPCNT      (1 << 23)
+
+/* xgetbv bit flags */
+#define XSTATE_SSE (1 << 1)
+#define XSTATE_YMM (1 << 2)
+#define XSTATE_ZMM (7 << 5)
+
+namespace {
+
+void run_cpuid(int eax, int ecx, int* abcd)
+{
+#if defined(_MSC_VER)
+  __cpuidex(abcd, eax, ecx);
+#else
+  int ebx = 0;
+  int edx = 0;
+
+  #if defined(__i386__) && \
+      defined(__PIC__)
+    /* in case of PIC under 32-bit EBX cannot be clobbered */
+    __asm__ ("movl %%ebx, %%edi;"
+             "cpuid;"
+             "xchgl %%ebx, %%edi;"
+             : "=D" (ebx),
+               "+a" (eax),
+               "+c" (ecx),
+               "=d" (edx));
+  #else
+    __asm__ ("cpuid;"
+             : "+b" (ebx),
+               "+a" (eax),
+               "+c" (ecx),
+               "=d" (edx));
   #endif
+
+  abcd[0] = eax;
+  abcd[1] = ebx;
+  abcd[2] = ecx;
+  abcd[3] = edx;
+#endif
+}
+
+// Get Value of Extended Control Register
+int get_xcr0()
+{
+  int xcr0;
+
+#if defined(_MSC_VER)
+  xcr0 = (int) _xgetbv(0);
+#else
+  __asm__ ("xgetbv" : "=a" (xcr0) : "c" (0) : "%edx" );
+#endif
+
+  return xcr0;
+}
+
+bool has_AVX512()
+{
+  int abcd[4];
+
+  run_cpuid(1, 0, abcd);
+
+  // PrimeGenerator::fillNextPrimes() requires POPCNT
+  if ((abcd[2] & bit_POPCNT) != bit_POPCNT)
+    return false;
+
+  int osxsave_mask = (1 << 27);
+
+  // Ensure OS supports extended processor state management
+  if ((abcd[2] & osxsave_mask) != osxsave_mask)
+    return false;
+
+  int ymm_mask = XSTATE_SSE | XSTATE_YMM;
+  int zmm_mask = XSTATE_SSE | XSTATE_YMM | XSTATE_ZMM;
+
+  int xcr0 = get_xcr0();
+
+  // Check AVX OS support
+  if ((xcr0 & ymm_mask) != ymm_mask)
+    return false;
+
+  // Check AVX512 OS support
+  if ((xcr0 & zmm_mask) != zmm_mask)
+    return false;
+
+  run_cpuid(7, 0, abcd);
+
+  // PrimeGenerator::fillNextPrimes() requires AVX512F, AVX512VBMI & AVX512VBMI2
+  return ((abcd[1] & bit_AVX512F) == bit_AVX512F &&
+          (abcd[2] & (bit_AVX512VBMI | bit_AVX512VBMI2)) == (bit_AVX512VBMI | bit_AVX512VBMI2));
+}
+
+} // namespace
+
 #endif
 
 #if defined(_WIN32)
@@ -50,133 +161,47 @@ using std::size_t;
 #include <iterator>
 #include <map>
 
-#if defined(__i386__) || \
-    defined(_M_IX86) || \
-    defined(__x86_64__) || \
-    defined(_M_X64) || \
-    defined(_M_AMD64)
-  #define IS_X86
-#endif
-
-#if defined(IS_X86)
-
-#include <algorithm>
-
-// Check if compiler supports <intrin.h>
-#if defined(__has_include)
-  #if __has_include(<intrin.h>)
-    #define HAS_INTRIN_H
-  #endif
-#elif defined(_MSC_VER)
-  #define HAS_INTRIN_H
-#endif
-
-// Check if compiler supports CPUID
-#if defined(HAS_INTRIN_H)
-  #include <intrin.h>
-  #define MSVC_CPUID
-#elif defined(__GNUC__) || \
-      defined(__clang__)
-  #define GNUC_CPUID
-#endif
-
-namespace {
-
-/// CPUID is not portable across all x86 CPU vendors and there
-/// are many pitfalls. For this reason we prefer to get CPU
-/// information from the operating system instead of CPUID.
-/// We only use CPUID for getting the CPU name on Windows x86
-/// because there is no other way to get that information.
-///
-void cpuId(int cpuInfo[4], int eax)
-{
-#if defined(MSVC_CPUID)
-  __cpuid(cpuInfo, eax);
-#elif defined(GNUC_CPUID)
-  int ebx = 0;
-  int ecx = 0;
-  int edx = 0;
-
-  #if defined(__i386__) && \
-      defined(__PIC__)
-    // in case of PIC under 32-bit EBX cannot be clobbered
-    __asm__ ("movl %%ebx, %%edi;"
-             "cpuid;"
-             "xchgl %%ebx, %%edi;"
-             : "+a" (eax),
-               "=D" (ebx),
-               "=c" (ecx),
-               "=d" (edx));
-  #else
-    __asm__ ("cpuid;"
-             : "+a" (eax),
-               "=b" (ebx),
-               "=c" (ecx),
-               "=d" (edx));
-  #endif
-
-  cpuInfo[0] = eax;
-  cpuInfo[1] = ebx;
-  cpuInfo[2] = ecx;
-  cpuInfo[3] = edx;
-#else
-  // CPUID is not supported
-  eax = 0;
-
-  cpuInfo[0] = eax;
-  cpuInfo[1] = 0;
-  cpuInfo[2] = 0;
-  cpuInfo[3] = 0;
-#endif
-}
-
-/// Remove all leading and trailing
-/// space characters.
-///
-void trimString(std::string& str)
-{
-  std::string spaceChars = " \f\n\r\t\v";
-  size_t pos = str.find_first_not_of(spaceChars);
-  str.erase(0, pos);
-
-  pos = str.find_last_not_of(spaceChars);
-  if (pos != std::string::npos)
-    str.erase(pos + 1);
-}
-
-} // namespace
-
-#endif
-
 namespace {
 
 std::string getCpuName()
 {
   std::string cpuName;
 
-#if defined(IS_X86)
+#if defined(HAS_CPUID)
   // Get the CPU name using CPUID.
   // Example: Intel(R) Core(TM) i7-6700 CPU @ 3.40GHz
   // https://en.wikipedia.org/wiki/CPUID
 
   int cpuInfo[4] = { 0, 0, 0, 0 };
-  cpuId(cpuInfo, 0x80000000);
+  run_cpuid(0x80000000, 0, cpuInfo);
   std::vector<int> vect;
 
   // check if CPU name is supported
   if ((unsigned) cpuInfo[0] >= 0x80000004u)
   {
-    cpuId(cpuInfo, 0x80000002);
+    run_cpuid(0x80000002, 0, cpuInfo);
     std::copy_n(cpuInfo, 4, std::back_inserter(vect));
 
-    cpuId(cpuInfo, 0x80000003);
+    run_cpuid(0x80000003, 0, cpuInfo);
     std::copy_n(cpuInfo, 4, std::back_inserter(vect));
 
-    cpuId(cpuInfo, 0x80000004);
+    run_cpuid(0x80000004, 0, cpuInfo);
     std::copy_n(cpuInfo, 4, std::back_inserter(vect));
 
     vect.push_back(0);
     cpuName = (char*) vect.data();
+
+    auto trimString = [](std::string& str)
+    {
+      std::string spaceChars = " \f\n\r\t\v";
+      std::size_t pos = str.find_first_not_of(spaceChars);
+      str.erase(0, pos);
+
+      pos = str.find_last_not_of(spaceChars);
+      if (pos != std::string::npos)
+        str.erase(pos + 1);
+    };
+
     trimString(cpuName);
   }
 #endif
@@ -224,17 +249,17 @@ void CpuInfo::init()
       cacheSizes{0, 0, 0, 0},
       cacheSharing{0, 0, 0, 0}
     { }
-    std::array<size_t, 4> cacheSizes;
-    std::array<size_t, 4> cacheSharing;
+    pod_array<size_t, 4> cacheSizes;
+    pod_array<size_t, 4> cacheSharing;
   };
 
   struct L1CacheStatistics
   {
     long cpuCoreId = -1;
-    size_t cpuCoreCount = 0;
+    std::size_t cpuCoreCount = 0;
   };
 
-  using CacheSize_t = size_t;
+  using CacheSize_t = std::size_t;
   // Items must be sorted in ascending order
   std::map<CacheSize_t, L1CacheStatistics> l1CacheStatistics;
   std::vector<CpuCoreCacheInfo> cacheInfo;
@@ -243,7 +268,7 @@ void CpuInfo::init()
 
   // Fill the cacheInfo vector with the L1, L2 & L3 cache
   // sizes and cache sharing of each CPU core.
-  for (size_t i = 0; i < bytes; i += info->Size)
+  for (std::size_t i = 0; i < bytes; i += info->Size)
   {
     info = (SYSTEM_LOGICAL_PROCESSOR_INFORMATION_EX*) &buffer[i];
 
@@ -253,8 +278,8 @@ void CpuInfo::init()
         (info->Cache.Type == CacheData ||
          info->Cache.Type == CacheUnified))
     {
-      size_t cpuCoreIndex = 0;
-      size_t cacheSharing = 0;
+      std::size_t cpuCoreIndex = 0;
+      std::size_t cacheSharing = 0;
 
       // Cache.GroupMask.Mask contains one bit set for
       // each logical CPU core sharing the cache.
@@ -281,7 +306,7 @@ void CpuInfo::init()
         // filled (they may have less than maxCpusPerProcessorGroup
         // CPU cores). However our formula yields unique
         // cpuCoreIds which is good enough for our usage.
-        size_t cpuCoreId = processorGroup * maxCpusPerProcessorGroup + cpuCoreIndex;
+        std::size_t cpuCoreId = processorGroup * maxCpusPerProcessorGroup + cpuCoreIndex;
 
         if (cacheInfo.size() <= cpuCoreId)
           cacheInfo.resize((cpuCoreId + 1) * 2);
@@ -344,14 +369,14 @@ void CpuInfo::init()
   if (!bytes)
     return;
 
-  size_t threadsPerCore = 0;
-  size_t size = bytes / sizeof(SYSTEM_LOGICAL_PROCESSOR_INFORMATION);
+  std::size_t threadsPerCore = 0;
+  std::size_t size = bytes / sizeof(SYSTEM_LOGICAL_PROCESSOR_INFORMATION);
   std::vector<SYSTEM_LOGICAL_PROCESSOR_INFORMATION> info(size);
 
   if (!glpi(&info[0], &bytes))
     return;
 
-  for (size_t i = 0; i < size; i++)
+  for (std::size_t i = 0; i < size; i++)
   {
     if (info[i].Relationship == RelationProcessorCore)
     {
@@ -366,7 +391,7 @@ void CpuInfo::init()
     }
   }
 
-  for (size_t i = 0; i < size; i++)
+  for (std::size_t i = 0; i < size; i++)
   {
     if (info[i].Relationship == RelationCache &&
         info[i].Cache.Level >= 1 &&
@@ -402,9 +427,6 @@ void CpuInfo::init()
 #elif defined(APPLE_SYSCTL)
 
 #include <primesieve/pmath.hpp>
-
-#include <algorithm>
-#include <cstddef>
 #include <sys/sysctl.h>
 
 namespace {
@@ -417,11 +439,11 @@ template <typename T>
 std::vector<T> getSysctl(const std::string& name)
 {
   std::vector<T> res;
-  size_t bytes = 0;
+  std::size_t bytes = 0;
 
   if (!sysctlbyname(name.data(), 0, &bytes, 0, 0))
   {
-    size_t size = ceilDiv(bytes, sizeof(T));
+    std::size_t size = ceilDiv(bytes, sizeof(T));
     std::vector<T> buffer(size, 0);
     if (!sysctlbyname(name.data(), buffer.data(), &bytes, 0, 0))
       res = buffer;
@@ -453,12 +475,12 @@ void CpuInfo::init()
 
   // https://developer.apple.com/library/content/releasenotes/Performance/RN-AffinityAPI/index.html
   auto cacheSizes = getSysctl<size_t>("hw.cachesize");
-  for (size_t i = 1; i < std::min(cacheSizes.size(), cacheSizes_.size()); i++)
+  for (std::size_t i = 1; i < std::min(cacheSizes.size(), cacheSizes_.size()); i++)
     cacheSizes_[i] = cacheSizes[i];
 
   // https://developer.apple.com/library/content/releasenotes/Performance/RN-AffinityAPI/index.html
   auto cacheConfig = getSysctl<size_t>("hw.cacheconfig");
-  for (size_t i = 1; i < std::min(cacheConfig.size(), cacheSharing_.size()); i++)
+  for (std::size_t i = 1; i < std::min(cacheConfig.size(), cacheSharing_.size()); i++)
     cacheSharing_[i] = cacheConfig[i];
 }
 
@@ -468,9 +490,10 @@ void CpuInfo::init()
 
 #include <primesieve/primesieve_error.hpp>
 
-#include <algorithm>
 #include <cctype>
 #include <fstream>
+#include <iterator>
+#include <map>
 #include <set>
 #include <sstream>
 
@@ -484,7 +507,7 @@ namespace {
 void trimString(std::string& str)
 {
   std::string spaceChars = " \f\n\r\t\v";
-  size_t pos = str.find_first_not_of(spaceChars);
+  std::size_t pos = str.find_first_not_of(spaceChars);
   str.erase(0, pos);
 
   pos = str.find_last_not_of(spaceChars);
@@ -508,7 +531,7 @@ std::string getString(const std::string& filename)
 size_t getValue(const std::string& filename)
 {
   std::string str = getString(filename);
-  size_t val = 0;
+  std::size_t val = 0;
 
   if (!str.empty())
     val = std::stoul(str);
@@ -519,7 +542,7 @@ size_t getValue(const std::string& filename)
 size_t getCacheSize(const std::string& filename)
 {
   std::string str = getString(filename);
-  size_t val = 0;
+  std::size_t val = 0;
 
   if (!str.empty())
   {
@@ -561,7 +584,7 @@ std::string getCpuName(const std::string& line)
     "cpu"
   };
 
-  size_t pos = line.find(':');
+  std::size_t pos = line.find(':');
   std::string cpuName;
 
   if (pos != std::string::npos)
@@ -593,7 +616,7 @@ std::string getCpuName()
   if (file)
   {
     std::string line;
-    size_t i = 0;
+    std::size_t i = 0;
 
     while (std::getline(file, line))
     {
@@ -630,7 +653,7 @@ std::vector<std::string> split(const std::string& str,
 ///
 size_t parseThreadList(const std::string& filename)
 {
-  size_t threads = 0;
+  std::size_t threads = 0;
   auto threadList = getString(filename);
   auto tokens = split(threadList, ',');
 
@@ -658,7 +681,7 @@ size_t parseThreadList(const std::string& filename)
 ///
 size_t parseThreadMap(const std::string& filename)
 {
-  size_t threads = 0;
+  std::size_t threads = 0;
   std::string threadMap = getString(filename);
 
   for (char c : threadMap)
@@ -666,7 +689,7 @@ size_t parseThreadMap(const std::string& filename)
     if (c != ',')
     {
       std::string hexChar { c };
-      size_t bitmap = std::stoul(hexChar, nullptr, 16);
+      std::size_t bitmap = std::stoul(hexChar, nullptr, 16);
       for (; bitmap > 0; threads++)
         bitmap &= bitmap - 1;
     }
@@ -685,7 +708,7 @@ size_t parseThreadMap(const std::string& filename)
 size_t getThreads(const std::string& threadList,
                   const std::string& threadMap)
 {
-  size_t threads = parseThreadList(threadList);
+  std::size_t threads = parseThreadList(threadList);
 
   if (threads != 0)
     return threads;
@@ -701,35 +724,92 @@ void CpuInfo::init()
 {
   std::string cpusOnline = "/sys/devices/system/cpu/online";
   logicalCpuCores_ = parseThreadList(cpusOnline);
+  bool identicalL1CacheSizes = false;
 
-  // For hybrid CPUs with multiple types of CPU cores Linux seems
-  // to order the CPU cores within /sys/devices/system/cpu*
-  // from fastest to slowest. By picking a CPU core from the middle
-  // we hopefully get an average CPU core that is representative
-  // for the CPU's overall (multi-threading) performance.
-  std::string cpuNumber = std::to_string(logicalCpuCores_ / 2);
+  using CacheSize_t = std::size_t;
+  // Items must be sorted in ascending order
+  std::map<CacheSize_t, std::size_t> l1CacheStatistics;
+  std::vector<size_t> cpuIds;
+  cpuIds.reserve(3);
 
-  // Retrieve CPU cache info
-  for (size_t i = 0; i <= 3; i++)
+  // Check 1st, last & middle CPU core
+  cpuIds.push_back(0);
+  if (logicalCpuCores_ >= 2)
+    cpuIds.push_back(logicalCpuCores_ - 1);
+  if (logicalCpuCores_ >= 3)
+    cpuIds.push_back(logicalCpuCores_ / 2);
+
+  // Because of hybrid CPUs with big & little CPU cores we
+  // first check whether there are CPU cores with different
+  // L1 data cache sizes in the system. Because these
+  // checks are slow, we only check 3 different CPU cores.
+  for (std::size_t cpuId : cpuIds)
   {
-    std::string path = "/sys/devices/system/cpu/cpu" + cpuNumber + "/cache/index" + std::to_string(i);
-    std::string cacheLevel = path + "/level";
-    size_t level = getValue(cacheLevel);
-
-    if (level >= 1 &&
-        level <= 3)
+    for (std::size_t i = 0; i <= 3; i++)
     {
-      std::string type = path + "/type";
-      std::string cacheType = getString(type);
+      std::string path = "/sys/devices/system/cpu/cpu" + std::to_string(cpuId) + "/cache/index" + std::to_string(i);
+      std::string cacheLevel = path + "/level";
+      std::size_t level = getValue(cacheLevel);
 
-      if (cacheType == "Data" ||
-          cacheType == "Unified")
+      if (level == 1)
       {
-        std::string cacheSize = path + "/size";
-        std::string sharedCpuList = path + "/shared_cpu_list";
-        std::string sharedCpuMap = path + "/shared_cpu_map";
-        cacheSizes_[level] = getCacheSize(cacheSize);
-        cacheSharing_[level] = getThreads(sharedCpuList, sharedCpuMap);
+        std::string type = path + "/type";
+        std::string cacheType = getString(type);
+
+        if (cacheType == "Data" ||
+            cacheType == "Unified")
+        {
+          std::size_t cacheSize = getCacheSize(path + "/size");
+          if (cacheSize > 0)
+          {
+            if (l1CacheStatistics.find(cacheSize) == l1CacheStatistics.end())
+              l1CacheStatistics[cacheSize] = cpuId;
+            else
+              identicalL1CacheSizes = true;
+          }
+          break;
+        }
+      }
+    }
+
+    // We have found 2 identical CPU cores.
+    // In this case we assume all CPU cores
+    // have the same cache hierarchy.
+    if (identicalL1CacheSizes)
+      break;
+  }
+
+  // Retrieve the cache sizes of the CPU core with the middle
+  // L1 data cache size. If there are only 2 different L1
+  // cache sizes we retrieve the cache sizes of the CPU core
+  // with the smaller L1 data cache size.
+  if (!l1CacheStatistics.empty())
+  {
+    auto iter = l1CacheStatistics.begin();
+    std::advance(iter, (l1CacheStatistics.size() - 1) / 2);
+    std::size_t cpuId = iter->second;
+
+    for (std::size_t i = 0; i <= 3; i++)
+    {
+      std::string path = "/sys/devices/system/cpu/cpu" + std::to_string(cpuId) + "/cache/index" + std::to_string(i);
+      std::string cacheLevel = path + "/level";
+      std::size_t level = getValue(cacheLevel);
+
+      if (level >= 1 &&
+          level <= 3)
+      {
+        std::string type = path + "/type";
+        std::string cacheType = getString(type);
+
+        if (cacheType == "Data" ||
+            cacheType == "Unified")
+        {
+          std::string cacheSize = path + "/size";
+          std::string sharedCpuList = path + "/shared_cpu_list";
+          std::string sharedCpuMap = path + "/shared_cpu_map";
+          cacheSizes_[level] = getCacheSize(cacheSize);
+          cacheSharing_[level] = getThreads(sharedCpuList, sharedCpuMap);
+        }
       }
     }
   }
@@ -780,6 +860,19 @@ std::string CpuInfo::cpuName() const
   }
 }
 
+/// This method is only used by the primesieve command-line app
+/// with the --cpu-info option. Therefore we currently don't
+/// cache the result of has_AVX512().
+///
+bool CpuInfo::hasAVX512() const
+{
+  #if defined(HAS_CPUID)
+    return has_AVX512();
+  #else
+    return false;
+  #endif
+}
+
 size_t CpuInfo::logicalCpuCores() const
 {
   return logicalCpuCores_;
@@ -819,7 +912,6 @@ std::string CpuInfo::getError() const
 {
   return error_;
 }
-
 bool CpuInfo::hasCpuName() const
 {
   return !cpuName().empty();
