@@ -1,7 +1,7 @@
 ///
 /// @file  cmdoptions.hpp
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -10,12 +10,12 @@
 #ifndef CMDOPTIONS_HPP
 #define CMDOPTIONS_HPP
 
+#include <primesieve/pod_vector.hpp>
 #include <stdint.h>
-#include <deque>
 
 struct CmdOptions
 {
-  std::deque<uint64_t> numbers;
+  primesieve::pod_vector<uint64_t> numbers;
   int flags = 0;
   int sieveSize = 0;
   int threads = 0;
