@@ -26,7 +26,7 @@ const uint64_t small_primes[25] =
 };
 
 // primes inside [18446744073709550681, 18446744073709551533]
-const uint64_t large_primes[19] =
+const uint64_t large_primes[20] =
 {
   18446744073709550681ull,
   18446744073709550717ull,
@@ -46,7 +46,8 @@ const uint64_t large_primes[19] =
   18446744073709551427ull,
   18446744073709551437ull,
   18446744073709551521ull,
-  18446744073709551533ull
+  18446744073709551533ull,
+  18446744073709551557ull
 };
 
 void check(int OK)
@@ -96,9 +97,9 @@ int main()
   check(size == 0);
   primesieve_free(primes);
 
-  primes = (uint64_t*) primesieve_generate_primes(18446744073709550672ull, 18446744073709551556ull, &size, UINT64_PRIMES);
+  primes = (uint64_t*) primesieve_generate_primes(18446744073709550672ull, 18446744073709551615ull, &size, UINT64_PRIMES);
   printf("primes.size = %zu", size);
-  check(size == 19);
+  check(size == 20);
 
   for (i = 0; i < size; i++)
   {
@@ -107,6 +108,11 @@ int main()
   }
 
   primesieve_free(primes);
+
+  int32_t* primes32 = (int32_t*) primesieve_generate_primes((1ull << 31) - 1000, (1ull << 31) + 1000, &size, INT32_PRIMES);
+  printf("Detect 31-bit overflow:");
+  check(primes32 == NULL);
+
   printf("\n");
   printf("All tests passed successfully!\n");
 
