@@ -2,7 +2,7 @@
 /// @file   isqrt.cpp
 /// @brief  Test integer square root function.
 ///
-/// Copyright (C) 2017 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -15,13 +15,11 @@
 #include <cmath>
 #include <cstdlib>
 
-using namespace std;
-
 void check(bool OK)
 {
-  cout << "   " << (OK ? "OK" : "ERROR") << "\n";
+  std::cout << "   " << (OK ? "OK" : "ERROR") << "\n";
   if (!OK)
-    exit(1);
+    std::exit(1);
 }
 
 int main()
@@ -33,40 +31,40 @@ int main()
   for (n = 0; n < 100000; n++)
   {
     res1 = isqrt(n);
-    res2 = sqrt((double) n);
-    cout << "isqrt(" << n << ") = " << res1;
+    res2 = std::sqrt(n);
+    std::cout << "isqrt(" << n << ") = " << res1;
     check(res1 == (uint64_t) res2);
   }
 
   n = (1ull << 32) - 1;
   res1 = isqrt(n);
-  res2 = sqrt((double) n);
-  cout << "isqrt(" << n << ") = " << res1;
+  res2 = std::sqrt(n);
+  std::cout << "isqrt(" << n << ") = " << res1;
   check(res1 == (uint64_t) res2);
 
   n = 1ull << 32;
   res1 = isqrt(n);
-  res2 = sqrt((double) n);
-  cout << "isqrt(" << n << ") = " << res1;
+  res2 = std::sqrt(n);
+  std::cout << "isqrt(" << n << ") = " << res1;
   check(res1 == (uint64_t) res2);
 
   n = 1000000000000000000ull - 1;
   res1 = isqrt(n);
-  cout << "isqrt(" << n << ") = " << res1;
+  std::cout << "isqrt(" << n << ") = " << res1;
   check(res1 == 999999999);
 
   n = 1000000000000000000ull;
   res1 = isqrt(n);
-  cout << "isqrt(" << n << ") = " << res1;
+  std::cout << "isqrt(" << n << ") = " << res1;
   check(res1 == 1000000000);
 
   n = 18446744073709551615ull;
   res1 = isqrt(n);
-  cout << "isqrt(" << n << ") = " << res1;
+  std::cout << "isqrt(" << n << ") = " << res1;
   check(res1 == 4294967295ull);
 
-  cout << endl;
-  cout << "All tests passed successfully!" << endl;
+  std::cout << std::endl;
+  std::cout << "All tests passed successfully!" << std::endl;
 
   return 0;
 }
